Attribute VB_Name = "modIcons"
'
'
' les structures et le principe proviennent de l'exemple iconpro (samples/sdk/win32)
'

Option Base 0
Option Explicit

Public Const LogFile = "\ExtractIcons.log"

' These first two structs represent how the icon information is stored
' when it is bound into a EXE or DLL file. Structure members are WORD
' aligned and the last member of the structure is the ID instead of
' the imageoffset.
Public Type MEMICONDIRENTRY
    bWidth          As Byte         ' Width of the image
    bHeight         As Byte         ' Height of the image (times 2)
    bColorCount     As Byte         ' Number of colors in image (0 if >=8bpp)
    bReserved       As Byte         ' Reserved
    wPlanes         As Integer      ' Color Planes
    wBitCount       As Integer      ' Bits per pixel
    dwBytesInRes    As Long         ' how many bytes in this resource?
    nID             As Integer      ' the ID
End Type
Public Type MEMICONDIR
    idReserved      As Integer      ' Reserved
    idType          As Integer      ' resource type (1 for icons)
    idCount         As Integer      ' how many images?
End Type


' These next two structs represent how the icon information is stored
' in an ICO file.
Public Type ICONDIRENTRY
    bWidth          As Byte         ' Width of the image
    bHeight         As Byte         ' Height of the image (times 2)
    bColorCount     As Byte         ' Number of colors in image (0 if >=8bpp)
    bReserved       As Byte         ' Reserved
    wPlanes         As Integer      ' Color Planes
    wBitCount       As Integer      ' Bits per pixel
    dwBytesInRes    As Long         ' how many bytes in this resource?
    dwImageOffset   As Long         ' where in the file is this image
End Type
Public Type ICONDIR
    idReserved      As Integer      ' Reserved
    idType          As Integer      ' resource type (1 for icons)
    idCount         As Integer      ' how many images?
End Type


' The following two structs are for the use of this program in
' manipulating icons. They are more closely tied to the operation
' of this program than the structures listed above. One of the
' main differences is that they provide a pointer to the DIB
' information of the masks.
Private Type ICONIMAGE
    Width       As Integer
    Height      As Integer
    Colors      As Integer                  ' Width, Height and bpp
    lpBits()    As Byte                     ' ptr to DIB bits
    dwNumBytes  As Long                     ' how many bytes?
    lpbi        As BITMAPINFO               ' ptr to header
    lpXOR()     As Byte                     ' ptr to XOR image bits
    lpAND()     As Byte                     ' ptr to AND image bits
End Type

Private Type ICONRESOURCE
    bHasChanged     As Long                     ' Has image changed?
    szOriginalICOFileName(MAX_PATH) As Byte     ' Original name
    szOriginalDLLFileName(MAX_PATH) As Byte     ' Original name
    nNumImages      As Integer                  ' How many images?
    IconImages()    As ICONIMAGE                ' Image entries
End Type


Public Declare Sub MoveMemory Lib "kernel32" Alias "RtlMoveMemory" (ByRef Dest As Any, ByRef Source As Any, ByVal Length As Long)
'Private Declare Sub MoveMemoryRV Lib "kernel32" Alias "RtlMoveMemory" (ByRef Dest As Any, ByVal Source As Any, ByVal Length As Long)
'Private Declare Sub MoveMemoryRR Lib "kernel32" Alias "RtlMoveMemory" (ByRef Dest As Any, ByRef Source As Any, ByVal Length As Long)


Public Extracted    As String
Public nIcons       As Integer

Private Function ResourceToICO(ByVal hModule As Long, ByVal lpIcon As Long, ByVal Length As Long)
    Dim lpIconMID   As MEMICONDIR
    Dim lpIR        As ICONRESOURCE
    Dim i           As Integer
    Dim hRsrc       As Long
    Dim lpIconMIDE  As MEMICONDIRENTRY
    Dim hGlobal     As Long
    
    MoveMemory lpIconMID, ByVal lpIcon, LenB(lpIconMID)
    lpIR.nNumImages = lpIconMID.idCount
'    lpIR.szOriginalDLLFileName = ""
'    lpIR.szOriginalICOFileName = ""
    ReDim lpIR.IconImages(lpIR.nNumImages)

    ' Loop through the images
    For i = 0 To lpIR.nNumImages - 1
        
        ' Get the individual image
        MoveMemory lpIconMIDE, ByVal lpIcon + LenB(lpIconMID) + i * LenB(lpIconMIDE), LenB(lpIconMIDE)
        
        hRsrc = FindResource(hModule, lpIconMIDE.nID, RT_ICON)
        If hRsrc = 0& Then
            Exit Function
        End If
        hGlobal = LoadResource(hModule, hRsrc)
        If hGlobal = 0& Then
            Exit Function
        End If
        
        lpIR.IconImages(i).dwNumBytes = SizeofResource(hModule, hRsrc)
        ReDim lpIR.IconImages(i).lpBits(lpIR.IconImages(i).dwNumBytes)
        MoveMemory lpIR.IconImages(i).lpBits(0), ByVal LockResource(hGlobal), lpIR.IconImages(i).dwNumBytes
        AdjustIconImagePointers lpIR.IconImages(i)
        
    Next
    
    nIcons = nIcons + 1
    WriteIconToICOFile lpIR, App.path & "\icon" & Right("0000" & nIcons, 4) & ".ico"
End Function

Private Function AdjustIconImagePointers(ByRef lpImage As ICONIMAGE) As Long
        
    ' BITMAPINFO is at beginning of bits
    MoveMemory lpImage.lpbi, lpImage.lpBits(0), LenB(lpImage.lpbi)
    ' Width - simple enough
    lpImage.Width = lpImage.lpbi.bmiHeader.biWidth
    ' Icons are stored in funky format where height is doubled - account for it
    lpImage.Height = (lpImage.lpbi.bmiHeader.biHeight) / 2
    ' How many colors?
    lpImage.Colors = lpImage.lpbi.bmiHeader.biPlanes * lpImage.lpbi.bmiHeader.biBitCount
    ' XOR bits follow the header and color table
'    lpImage.lpXOR = FindDIBBits((LPSTR)lpImage.lpbi)
    ' AND bits follow the XOR bits
'    lpImage.lpAND = lpImage.lpXOR + (lpImage.Height*BytesPerLine((LPBITMAPINFOHEADER)(lpImage->lpbi)))
    
    AdjustIconImagePointers = True
End Function

Private Function WriteIconToICOFile(ByRef lpIR As ICONRESOURCE, ByRef szFileName As String) As Long
    Dim hFile           As Long
    Dim i               As Integer
    Dim l               As Long
    Dim dwBytesWritten  As Long
    
    hFile = CreateFile(szFileName, GENERIC_WRITE, 0, 0&, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0&)
    If hFile = INVALID_HANDLE_VALUE Then
        l = GetLastError()
        Debug.Print "getLastError()="; l
        MsgBox "Error Opening File for Writing: " & szFileName, vbOK
        Exit Function
    End If

    If Not WriteICOHeader(hFile, lpIR.nNumImages) Then
        MsgBox "Error Writing ICO File: " & szFileName, vbOK
        CloseHandle hFile
        Exit Function
    End If

    ' Write the ICONDIRENTRY's
    For i = 0 To lpIR.nNumImages - 1
        Dim ide As ICONDIRENTRY

        ' Convert internal format to ICONDIRENTRY
        ide.bWidth = lpIR.IconImages(i).Width
        ide.bHeight = lpIR.IconImages(i).Height
        ide.bReserved = 0
        ide.wPlanes = lpIR.IconImages(i).lpbi.bmiHeader.biPlanes
        ide.wBitCount = lpIR.IconImages(i).lpbi.bmiHeader.biBitCount
        If (ide.wPlanes * ide.wBitCount) >= 8 Then
            ide.bColorCount = 0
        Else
            ide.bColorCount = shl(1, (ide.wPlanes * ide.wBitCount))
        End If
        ide.dwBytesInRes = lpIR.IconImages(i).dwNumBytes
        ide.dwImageOffset = CalculateImageOffset(lpIR, i)

        ' Write the ICONDIRENTRY out to disk
        If 0 = WriteFile(hFile, ide, LenB(ide), dwBytesWritten, 0&) Then
            Exit Function
        End If
        ' Did we write a full ICONDIRENTRY ?
        If dwBytesWritten <> LenB(ide) Then
            Exit Function
        End If
    Next
    
    ' Write the image bits for each image
    For i = 0 To lpIR.nNumImages - 1
        Dim dwTemp  As Long
        
        dwTemp = lpIR.IconImages(i).lpbi.bmiHeader.biSizeImage

        ' Set the sizeimage member to zero
        lpIR.IconImages(i).lpbi.bmiHeader.biSizeImage = 0
        ' Write the image bits to file
        If 0 = WriteFile(hFile, lpIR.IconImages(i).lpBits(0), lpIR.IconImages(i).dwNumBytes, dwBytesWritten, 0&) Then
            Exit Function
        End If
        If dwBytesWritten <> lpIR.IconImages(i).dwNumBytes Then
            Exit Function
        End If
        ' set it back
        lpIR.IconImages(i).lpbi.bmiHeader.biSizeImage = dwTemp
    Next
    
    CloseHandle hFile
        
End Function

Private Function CalculateImageOffset(ByRef lpIR As ICONRESOURCE, ByVal nIndex As Integer) As Long
    Dim ide     As ICONDIRENTRY
    Dim dwSize  As Long
    Dim i       As Integer

    ' Calculate the ICO header size
    dwSize = 3 * 2
    ' Add the ICONDIRENTRY's
    dwSize = dwSize + lpIR.nNumImages * LenB(ide)
    ' Add the sizes of the previous images
    For i = 0 To nIndex - 1
        dwSize = dwSize + lpIR.IconImages(i).dwNumBytes
    Next
    ' we're there - return the number
    CalculateImageOffset = dwSize
End Function

Private Function WriteICOHeader(ByVal hFile As Long, ByVal nNumEntries As Integer) As Long
    Dim Output          As Integer
    Dim dwBytesWritten  As Long
    
    WriteICOHeader = False
    
    ' Write 'reserved' WORD
    Output = 0
    If 0 = WriteFile(hFile, Output, 2, dwBytesWritten, 0&) Then
        Debug.Print "GetLastError()="; GetLastError()
        Exit Function
    End If
    ' Did we write a WORD?
    If dwBytesWritten <> 2 Then
        Exit Function
    End If
    ' Write 'type' WORD (1)
    Output = 1
    If 0 = WriteFile(hFile, Output, 2, dwBytesWritten, 0&) Then
        Exit Function
    End If
    ' Did we write a WORD?
    If dwBytesWritten <> 2 Then
        Exit Function
    End If
    ' Write Number of Entries
    Output = nNumEntries
    If 0 = WriteFile(hFile, Output, 2, dwBytesWritten, 0&) Then
        Exit Function
    End If
    ' Did we write a WORD?
    If dwBytesWritten <> 2 Then
        Exit Function
    End If
    
    WriteICOHeader = True
End Function

Private Function shl(ByVal a As Long, ByVal i As Integer) As Long
    Do While i > 0
        a = a * 2
        i = i - 1
    Loop
    shl = a
End Function


Private Function EnumResNameProc(ByVal hModule As Long, ByVal lpszType As Long, ByVal lpszName As Long, lParam As Long) As Long
    Dim hRsrc       As Long
    Dim hGlobal     As Long
    Dim lpIcon      As Long
'    Dim hIcon       As Long

    hRsrc = FindResource(hModule, lpszName, lpszType)
    If hRsrc Then
        hGlobal = LoadResource(hModule, hRsrc)
        lpIcon = LockResource(hGlobal)
'        hIcon = CreateIconFromResource(lpIcon, SizeofResource(hModule, hRsrc), 1, &H30000)
        ResourceToICO hModule, lpIcon, SizeofResource(hModule, hRsrc)
    End If
    
    EnumResNameProc = 1
End Function

Private Function EnumResTypeProc(ByVal hModule As Long, ByVal lpszType As Long, lParam As Long) As Long
    Dim l   As Long
    
    If lpszType = RT_GROUP_ICON Then
        l = EnumResourceNames(hModule, lpszType, AddressOf EnumResNameProc, 0&)
    End If
    EnumResTypeProc = 1
End Function

Public Sub GetExtracted()
    Dim s   As String
    Dim i   As Integer
    
    On Error Resume Next
    Open App.path & LogFile For Input As #1
    i = Err.Number
    On Error GoTo 0
    If i = 0 Then
        Do Until EOF(1)
            Line Input #1, s
            Extracted = Extracted & s
        Loop
        Close #1
        Kill App.path & LogFile
    Else
        Extracted = ""
    End If
End Sub

Public Sub ExtractIcons(ByRef szFileName As String)
    Dim hModule     As Long
    Dim l           As Long
    Dim fd          As Integer
    
    fd = FreeFile()
    Open App.path & LogFile For Append As #fd
    Print #fd, szFileName
    Close #fd
        
    If InStr(Extracted, szFileName) Then Exit Sub
    
    ' essaie de sauter les dll  problme (i.e. qui font planter NT)
    If InStr(LCase(szFileName), "framebuf.dll") > 0 Then Exit Sub
    If InStr(LCase(szFileName), "lights.exe") > 0 Then Exit Sub
    If InStr(LCase(szFileName), "mga") > 0 Then Exit Sub
    If InStr(LCase(szFileName), "vga") > 0 Then Exit Sub
    If InStr(LCase(szFileName), "wow") > 0 Then Exit Sub
    If InStr(LCase(szFileName), "mcdsrv") > 0 Then Exit Sub
    If InStr(LCase(szFileName), "s3virge") > 0 Then Exit Sub
    If InStr(LCase(szFileName), "winsrv.dll") > 0 Then Exit Sub
    If InStr(LCase(szFileName), "hypertrm.dll") > 0 Then Exit Sub
    If InStr(LCase(szFileName), "wkssvc.dll") > 0 Then Exit Sub
    
    hModule = LoadLibrary(szFileName)
    If hModule = 0 Then Debug.Print "Erreur LoadLibrary(" & szFileName & ")": Exit Sub

    l = EnumResourceTypes(hModule, AddressOf EnumResTypeProc, 0&)

    l = FreeLibrary(hModule)
End Sub
